//---------------------------------------------------------------------------

#ifndef UnitDirect3D_1H
#define UnitDirect3D_1H
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <CheckLst.hpp>
#include <ExtCtrls.hpp>
#include <vector.h>
#include <d3d9.h>

//-----------------------------------------------------------------------
// Konstanten, Datentypen, Klassen
//-----------------------------------------------------------------------
struct Direct3DParameters
{
  int adapter;
  D3DDEVTYPE deviceType;
  bool windowed;
  D3DDISPLAYMODE displayMode;
  D3DFORMAT backBufferFormat;
  D3DFORMAT zStencilBufferFormat;
  D3DMULTISAMPLE_TYPE multiSampleType;
  DWORD multiSampleQualityLevel;
  DWORD flags;
  D3DCAPS9 deviceCaps;
};

struct FormatName
{
  AnsiString name;
  D3DFORMAT format;
};

struct MultiSampleTypeName
{
  AnsiString name;
  D3DMULTISAMPLE_TYPE type;
};

class Direct3DSetup
{
  private:
    LPDIRECT3D9 lpDirect3D;

  public:
    int nrAdapters;
    int adapter; //Adapter-ID
    D3DADAPTER_IDENTIFIER9* lpAdapters;

    D3DCAPS9 deviceCaps;
    int nrDeviceTypes;
    D3DDEVTYPE deviceType; //Gertetyp
    bool deviceTypeHAL;
    bool deviceTypeREF;

    int nrModes;
    int mode; //Nr des Videomodus (Genauer ber lpModes!)
    D3DDISPLAYMODE* lpModes;

    bool windowed; //Fenster-/Bildschirmmodus

    int nrMultiSampleTypeNames;
    MultiSampleTypeName multiSampleTypeNames[17];

    int nrFormatNames;
    FormatName formatNames[19];
    int nrBackBufferFormats;
    vector <FormatName> backBufferFormats;
    D3DFORMAT backBufferFormat; //Bildpufferformat

    int nrZStencilBufferFormats;
    vector <FormatName> zStencilBufferFormats;
    D3DFORMAT zStencilBufferFormat; //Z-Stencil-Pufferformat

    int nrMultiSampleTypes;
    D3DMULTISAMPLE_TYPE multiSampleType; //Multisample-Typ
    vector <MultiSampleTypeName> multiSampleTypes;

    vector <DWORD> nrQualityLevels;
    DWORD qualityLevel; //Multisample-Qualittsstufe

  public:
    Direct3DSetup();
    ~Direct3DSetup();

    void getAdapters();
    void getDisplayModes();
    void getDeviceCaps();
    void getBackBufferFormats();
    void getZStencilBufferFormats();
    void getMultiSampleTypes();

    AnsiString getAdapterName(int i);
    AnsiString getDisplayModeName(int i);

  private:
    void initFormatNames();
    void initMultiSampleTypeNames();
};

class SetupControls
{
  private:
    TForm* lpForm;
    TComboBox* lpComboBoxAdapters;
    TComboBox* lpComboBoxModes;
    TPanel* lpPanelType;
    TRadioButton* lpRadioButtonsType[2];
    TPanel *lpPanelWindowed;
    TRadioButton* lpRadioButtonsWindowed[2];
    TComboBox* lpComboBoxBackBuffer;
    TComboBox* lpComboBoxZStencilBuffer;
    TComboBox* lpComboBoxMultiSampleType;
    TComboBox* lpComboBoxMultiSampleQuality;

    Direct3DSetup* lpDirect3DSetup;
  public:
    SetupControls(Direct3DSetup* newDirect3DSetup,
                  TForm* newForm,
                  TComboBox* newComboBoxAdapters,
                  TComboBox* newComboBoxModes, TPanel* newPanelType,
                  TPanel* newPanelWindowed,
                  TComboBox* newComboBoxBackBuffer,
                  TComboBox* newComboBoxZStencil,
                  TComboBox* newComboBoxMultiSampleType,
                  TComboBox* newComboBoxMultiSampleQuality);

    void init();
    void refreshAdapters();
    void refreshType();
    void refreshWindowed();
    void refreshModes();
    void refreshBackBufferFormats();
    void refreshZStencilBufferFormats();
    void refreshMultiSampleTypes();
    void refreshMultiSampleQuality();

    void updateAdapter();
    void updateType();
    void updateMode();
    void updateWindowed();
    void updateBackBufferFormat();
    void updateZStencilBufferFormat();
    void updateMultiSampleType();
    void updateMultiSampleQuality();

    Direct3DParameters execute();
};

//-----------------------------------------------------------------------
// Klassendefinition frs Einstellungsfenster
//-----------------------------------------------------------------------

class TFormDirect3DSetup : public TForm
{
__published:	// Von der IDE verwaltete Komponenten
        TLabel *Label1;
        TComboBox *ComboBoxAdapters;
        TLabel *Label2;
        TLabel *Label3;
        TComboBox *ComboBoxModes;
        TButton *ButtonLaunch;
        TPanel *PanelType;
        TRadioButton *RadioButtonHAL;
        TRadioButton *RadioButtonREF;
        TLabel *Label4;
        TPanel *PanelWindowed;
        TRadioButton *RadioButtonWindowed;
        TRadioButton *RadioButtonFullScreen;
        TLabel *Label5;
        TComboBox *ComboBoxBackBuffer;
        TLabel *Label6;
        TComboBox *ComboBoxZStencil;
        TLabel *Label7;
        TComboBox *ComboBoxSampleType;
        TLabel *Label8;
        TComboBox *ComboBoxSampleQuality;
        void __fastcall ComboBoxAdaptersChange(TObject *Sender);
        void __fastcall RadioButtonHALClick(TObject *Sender);
        void __fastcall ComboBoxModesChange(TObject *Sender);
        void __fastcall RadioButtonFullScreenClick(TObject *Sender);
        void __fastcall ComboBoxBackBufferChange(TObject *Sender);
        void __fastcall ComboBoxZStencilChange(TObject *Sender);
        void __fastcall ComboBoxSampleTypeChange(TObject *Sender);
        void __fastcall ComboBoxSampleQualityChange(TObject *Sender);
        void __fastcall FormCreate(TObject *Sender);
        void __fastcall FormDestroy(TObject *Sender);
        void __fastcall ButtonLaunchClick(TObject *Sender);
private:        // Anwender-Deklarationen
public:		// Anwender-Deklarationen
        __fastcall TFormDirect3DSetup(TComponent* Owner);
        Direct3DSetup* lpDirect3DSetup;
        SetupControls* lpSetupControls;
        Direct3DParameters execute(); //siehe weiter unten!
};

//---------------------------------------------------------------------------
extern PACKAGE TFormDirect3DSetup *FormDirect3DSetup;
//---------------------------------------------------------------------------

//-----------------------------------------------------------------------
// Konstruktoren, Klassenmethoden von Direct3DSetup
//-----------------------------------------------------------------------

Direct3DSetup::Direct3DSetup()
{
  lpDirect3D = Direct3DCreate9(D3D_SDK_VERSION);
  if(lpDirect3D)
  {
    adapter = -1;
    lpAdapters = NULL;

    deviceTypeHAL = false;
    deviceTypeREF = false;

    mode = -1;
    lpModes = NULL;

    windowed = false;

    initFormatNames();
    initMultiSampleTypeNames();

    nrBackBufferFormats = 0;
    backBufferFormat = D3DFMT_UNKNOWN;

    nrZStencilBufferFormats = 0;
    zStencilBufferFormat = D3DFMT_UNKNOWN;
  }
}

Direct3DSetup::~Direct3DSetup()
{
  if(lpDirect3D)
  {
    lpDirect3D->Release();
  }
  if(lpAdapters)
  {
    delete [] lpAdapters;
  }
  if(lpModes)
  {
    delete [] lpModes;
  }
}

void Direct3DSetup::getAdapters()
{
  int i;

  nrAdapters = lpDirect3D->GetAdapterCount();
  lpAdapters = new D3DADAPTER_IDENTIFIER9[nrAdapters];

  for(i=0; i<nrAdapters; i++)
  {
    lpDirect3D->GetAdapterIdentifier(i, 0, &lpAdapters[i]);
  }

  adapter = 0;
}

void Direct3DSetup::getDisplayModes()
{
  int i;

  if (lpModes)
  {
    delete [] lpModes;
  }

  if (windowed)
  {
    nrModes = 1;
    lpModes = new D3DDISPLAYMODE[1];
    lpDirect3D->GetAdapterDisplayMode(adapter, &lpModes[0]);
  }
  else
  {
    nrModes = lpDirect3D->GetAdapterModeCount(adapter, D3DFMT_R5G6B5);
    lpModes = new D3DDISPLAYMODE[nrModes];

    for(i=0; i<nrModes; i++)
    {
      lpDirect3D->EnumAdapterModes(adapter, D3DFMT_R5G6B5, i, &lpModes[i]);
    }
  }
  mode = 0;
}

void Direct3DSetup::getDeviceCaps()
{
  nrDeviceTypes = 0;

  if(SUCCEEDED(lpDirect3D->
     GetDeviceCaps(adapter, D3DDEVTYPE_REF,
                   &deviceCaps)))
  {
    deviceTypeREF = true;
    nrDeviceTypes++;
    deviceType = D3DDEVTYPE_REF;
  }
  if(SUCCEEDED(lpDirect3D->
     GetDeviceCaps(adapter, D3DDEVTYPE_HAL,
                   &deviceCaps)))
  {
    deviceTypeHAL = true;
    nrDeviceTypes++;
    deviceType = D3DDEVTYPE_HAL;
  }
}

void Direct3DSetup::getBackBufferFormats()
{
  nrBackBufferFormats = 0;
  backBufferFormats.clear();

  if (windowed)
  {
    //Falls windowed == true:
    //Das aktuell aktive Bildschirmformat ermitteln
    D3DDISPLAYMODE displayMode;
    lpDirect3D->GetAdapterDisplayMode(adapter, &displayMode);
    D3DFORMAT displayFormat = displayMode.Format;

    //Im windowed-Modus nur 1 Bildpuffer-Format mglich
    nrBackBufferFormats = 1;

    //Feld formatNames[] nach displayMode durchsuchen
    bool gefunden = false;
    for (int i=0; i<9 && !gefunden; i++)
    {
      if (formatNames[i].format == displayFormat)
      {
        backBufferFormats.push_back(formatNames[i]);
        gefunden = true;
      }
    }
  }
  else
  {
    //Falls windowed == false:
    //Alle Formate im Feld formatNames auf Verwendbarkeit testen
    for(int i=0; i<9; i++)
    {
      if(SUCCEEDED(lpDirect3D->CheckDeviceFormat(adapter,
           deviceType, lpModes[mode].Format, D3DUSAGE_RENDERTARGET,
           D3DRTYPE_SURFACE, formatNames[i].format)))
      {
        if(SUCCEEDED(lpDirect3D->CheckDeviceType(adapter,
           deviceType, lpModes[mode].Format, formatNames[i].format,
           windowed)))
        {
          backBufferFormats.push_back(formatNames[i]);
          nrBackBufferFormats++;
        }
      }
    }
  }
  backBufferFormat = backBufferFormats[0].format;
}

void Direct3DSetup::getZStencilBufferFormats()
{
  //Die verwendbaren Z-Stencil-Puffer-Formate suchen
  nrZStencilBufferFormats = 0;
  zStencilBufferFormats.clear();

  for(int i=9; i<18; i++)
  {
    if(SUCCEEDED(lpDirect3D->CheckDeviceFormat(adapter,
       deviceType, lpModes[mode].Format, D3DUSAGE_DEPTHSTENCIL,
       D3DRTYPE_SURFACE, formatNames[i].format)))
    {
      if(SUCCEEDED(lpDirect3D->CheckDepthStencilMatch(adapter,
         deviceType, lpModes[mode].Format, backBufferFormat,
         formatNames[i].format)))
      {
        zStencilBufferFormats.push_back(formatNames[i]);
        nrZStencilBufferFormats++;
      }
    }
  }
  zStencilBufferFormat = zStencilBufferFormats[0].format;
}

void Direct3DSetup::getMultiSampleTypes()
{
  int i;
  DWORD nr;

  nrMultiSampleTypes = 0;
  multiSampleTypes.clear();
  nrQualityLevels.clear();

  for(i=0; i<nrMultiSampleTypeNames; i++)
  {
    if(SUCCEEDED(lpDirect3D->CheckDeviceMultiSampleType(
       adapter, deviceType, backBufferFormat, windowed,
       multiSampleTypeNames[i].type,&nr)))
    {
      if(SUCCEEDED(lpDirect3D->CheckDeviceMultiSampleType(
         adapter, deviceType, zStencilBufferFormat, windowed,
         multiSampleTypeNames[i].type,&nr)))
      {
        multiSampleTypes.push_back(multiSampleTypeNames[i]);
        nrQualityLevels.push_back(nr);
        nrMultiSampleTypes++;
      }
    }
  }
  multiSampleType = multiSampleTypes[0].type;
  qualityLevel = 0;
}

AnsiString Direct3DSetup::getAdapterName(int i)
{
  if(i<nrAdapters)
  {
    return lpAdapters[i].Description;
  }
  else
  {
    return "Unbekannt";
  }
}

AnsiString Direct3DSetup::getDisplayModeName(int i)
{
  if(i<nrModes)
  {
    AnsiString str =
       IntToStr(lpModes[i].Width) + " x " +
       IntToStr(lpModes[i].Height) + " (" +
       IntToStr(lpModes[i].RefreshRate) + " Hz)";
    return str;
  }
  else
  {
    return "Unbekannt";
  }
}

void Direct3DSetup::initFormatNames()
{
  nrFormatNames = 18;
 //Oberflchenformate
  formatNames[0].format = D3DFMT_R8G8B8;
  formatNames[0].name = "R8G8B8 (24 bit)";
  formatNames[1].format = D3DFMT_A8R8G8B8;
  formatNames[1].name = "A8R8G8B8 (32 bit)";
  formatNames[2].format = D3DFMT_X8R8G8B8;
  formatNames[2].name = "X8R8G8B8 (32 bit)";
  formatNames[3].format = D3DFMT_R5G6B5;
  formatNames[3].name = "R5G6B5 (16 bit)";
  formatNames[4].format = D3DFMT_A1R5G5B5;
  formatNames[4].name = "A1R5G5B5 (16 bit)";
  formatNames[5].format = D3DFMT_X1R5G5B5;
  formatNames[5].name = "X1R5G5B5 (16 bit)";
  formatNames[6].format = D3DFMT_A4R4G4B4;
  formatNames[6].name = "A4R4G4B4 (16 bit)";
  formatNames[7].format = D3DFMT_A2R10G10B10;
  formatNames[7].name = "A2R10G10B10 (32 bit)";
  formatNames[8].format = D3DFMT_R3G3B2;
  formatNames[8].name = "R3G3B2 (8 bit)";
  //Z-Buffer-Oberflchenformate
  formatNames[9].format = D3DFMT_D32;
  formatNames[9].name = "D32 (32 bit)";
  formatNames[10].format = D3DFMT_D24X8;
  formatNames[10].name = "D24X8 (32 bit)";
  formatNames[11].format = D3DFMT_D16;
  formatNames[11].name = "D16 (16 bit)";
  formatNames[12].format = D3DFMT_D16_LOCKABLE;
  formatNames[12].name = "D16_LOCKABLE (16 bit)";
  formatNames[13].format = D3DFMT_D32F_LOCKABLE;
  formatNames[13].name = "D32F_LOCKABLE (32 bit)";
  //Stencil- und Z-Bufferformate
  formatNames[14].format = D3DFMT_D24S8;
  formatNames[14].name = "D24S8 (32 bit)";
  formatNames[15].format = D3DFMT_D24X4S4;
  formatNames[15].name = "D24X4S4 (32 bit)";
  formatNames[16].format = D3DFMT_D15S1;
  formatNames[16].name = "D15S1 (16 bit)";
  formatNames[17].format = D3DFMT_D24FS8;
  formatNames[17].name = "D24FS8 (32 bit)";
}

void Direct3DSetup::initMultiSampleTypeNames()
{
  nrMultiSampleTypeNames = 17;
  multiSampleTypeNames[0].type = D3DMULTISAMPLE_NONE;
  multiSampleTypeNames[0].name = "Kein Sampling";
  multiSampleTypeNames[1].type = D3DMULTISAMPLE_NONMASKABLE;
  multiSampleTypeNames[1].name =
     "Hardwarespezifisches Sampling";
  multiSampleTypeNames[2].type = D3DMULTISAMPLE_2_SAMPLES;
  multiSampleTypeNames[2].name = "2-faches Sampling";
  multiSampleTypeNames[3].type = D3DMULTISAMPLE_3_SAMPLES;
  multiSampleTypeNames[3].name = "3-faches Sampling";
  multiSampleTypeNames[4].type = D3DMULTISAMPLE_4_SAMPLES;
  multiSampleTypeNames[4].name = "4-faches Sampling";
  multiSampleTypeNames[5].type = D3DMULTISAMPLE_5_SAMPLES;
  multiSampleTypeNames[5].name = "5-faches Sampling";
  multiSampleTypeNames[6].type = D3DMULTISAMPLE_6_SAMPLES;
  multiSampleTypeNames[6].name = "6-faches Sampling";
  multiSampleTypeNames[7].type = D3DMULTISAMPLE_7_SAMPLES;
  multiSampleTypeNames[7].name = "7-faches Sampling";
  multiSampleTypeNames[8].type = D3DMULTISAMPLE_8_SAMPLES;
  multiSampleTypeNames[8].name = "8-faches Sampling";
  multiSampleTypeNames[9].type = D3DMULTISAMPLE_9_SAMPLES;
  multiSampleTypeNames[9].name = "9-faches Sampling";
  multiSampleTypeNames[10].type = D3DMULTISAMPLE_10_SAMPLES;
  multiSampleTypeNames[10].name = "10-faches Sampling";
  multiSampleTypeNames[11].type = D3DMULTISAMPLE_11_SAMPLES;
  multiSampleTypeNames[11].name = "11-faches Sampling";
  multiSampleTypeNames[12].type = D3DMULTISAMPLE_12_SAMPLES;
  multiSampleTypeNames[12].name = "12-faches Sampling";
  multiSampleTypeNames[13].type = D3DMULTISAMPLE_13_SAMPLES;
  multiSampleTypeNames[13].name = "13-faches Sampling";
  multiSampleTypeNames[14].type = D3DMULTISAMPLE_14_SAMPLES;
  multiSampleTypeNames[14].name = "14-faches Sampling";
  multiSampleTypeNames[15].type = D3DMULTISAMPLE_15_SAMPLES;
  multiSampleTypeNames[15].name = "15-faches Sampling";
  multiSampleTypeNames[16].type = D3DMULTISAMPLE_16_SAMPLES;
  multiSampleTypeNames[16].name = "16-faches Sampling";
}

//-----------------------------------------------------------------------
// Konstruktoren, Klassenmethoden von SetupControls
//-----------------------------------------------------------------------

SetupControls::SetupControls(Direct3DSetup* newDirect3DSetup,
                             TForm* newForm,
                             TComboBox* newComboBoxAdapters,
                             TComboBox* newComboBoxModes,
                             TPanel* newPanelType,
                             TPanel* newPanelWindowed,
                             TComboBox* newComboBoxBackBuffer,
                             TComboBox* newComboBoxZStencilBuffer,
                             TComboBox* newComboBoxMultiSampleType,
                             TComboBox* newComboBoxMultiSampleQuality)
{
  int i;

  lpDirect3DSetup = newDirect3DSetup;

  lpForm = newForm;

  lpComboBoxAdapters = newComboBoxAdapters;
  lpComboBoxModes = newComboBoxModes;
  lpPanelType = newPanelType;
  for(i=0; i<2; i++)
  {
    lpRadioButtonsType[i]=(TRadioButton*)(lpPanelType->Controls[i]);
  }
  lpPanelWindowed = newPanelWindowed;
  for(i=0; i<2; i++)
  {
    lpRadioButtonsWindowed[i]=
       (TRadioButton*)(lpPanelWindowed->Controls[i]);
  }
  lpComboBoxBackBuffer = newComboBoxBackBuffer;
  lpComboBoxZStencilBuffer = newComboBoxZStencilBuffer;
  lpComboBoxMultiSampleType = newComboBoxMultiSampleType;
  lpComboBoxMultiSampleQuality = newComboBoxMultiSampleQuality;
}

void SetupControls::init()
{
  refreshAdapters();
  refreshType();
  refreshWindowed();
  refreshModes();
  refreshBackBufferFormats();
  refreshZStencilBufferFormats();
  refreshMultiSampleTypes();
  refreshMultiSampleQuality();
  updateAdapter();
  updateType();
  updateWindowed();
  updateMode();
  updateBackBufferFormat();
  updateZStencilBufferFormat();
  updateMultiSampleType();
  updateMultiSampleQuality();
}

void SetupControls::refreshAdapters()
{
  int i;

  lpDirect3DSetup->getAdapters();

  lpComboBoxAdapters->Items->Clear();

  for(i=0; i<lpDirect3DSetup->nrAdapters; i++)
  {
    lpComboBoxAdapters->Items->Add(lpDirect3DSetup
       ->getAdapterName(i));
  }

  lpDirect3DSetup->adapter = 0;
  lpComboBoxAdapters->ItemIndex = 0;
}

void SetupControls::refreshType()
{
  int i;

  lpDirect3DSetup->getDeviceCaps();

  lpRadioButtonsType[0]->Enabled =
     lpDirect3DSetup->deviceTypeHAL;
  lpRadioButtonsType[1]->Enabled =
     lpDirect3DSetup->deviceTypeREF;

  if(lpDirect3DSetup->deviceTypeHAL)
  {
    lpDirect3DSetup->deviceType = D3DDEVTYPE_HAL;
    lpRadioButtonsType[0]->Checked = true;
  }
  else if (lpDirect3DSetup->deviceTypeREF)
  {
    lpDirect3DSetup->deviceType = D3DDEVTYPE_REF;
    lpRadioButtonsType[1]->Checked = true;
  }
}

void SetupControls::refreshWindowed()
{
  int i;

  if (lpDirect3DSetup->windowed)
  {
    i = 1;
  }
  else
  {
    i = 0;
  }
  lpRadioButtonsWindowed[i]->Checked = true;
}

void SetupControls::refreshModes()
{
  int i;

  lpDirect3DSetup->getDisplayModes();

  lpComboBoxModes->Items->Clear();

  for(i=0; i<lpDirect3DSetup->nrModes; i++)
  {
    lpComboBoxModes->Items->Add(lpDirect3DSetup
       ->getDisplayModeName(i));
  }

  lpDirect3DSetup->mode = 0;
  lpComboBoxModes->ItemIndex = 0;
}

void SetupControls::refreshBackBufferFormats()
{
  int i;

  lpDirect3DSetup->getBackBufferFormats();

  lpComboBoxBackBuffer->Items->Clear();

  for(i=0; i<lpDirect3DSetup->nrBackBufferFormats; i++)
  {
    lpComboBoxBackBuffer->Items->Add(lpDirect3DSetup
       ->backBufferFormats[i].name);
  }

  lpDirect3DSetup->backBufferFormat = lpDirect3DSetup
       ->backBufferFormats[0].format;
  lpComboBoxBackBuffer->ItemIndex = 0;
}

void SetupControls::refreshZStencilBufferFormats()
{
  int i;

  lpDirect3DSetup->getZStencilBufferFormats();

  lpComboBoxZStencilBuffer->Items->Clear();

  for(i=0; i<lpDirect3DSetup->nrZStencilBufferFormats; i++)
  {
    lpComboBoxZStencilBuffer->Items->Add(lpDirect3DSetup
       ->zStencilBufferFormats[i].name);
  }

  lpDirect3DSetup->zStencilBufferFormat = lpDirect3DSetup
       ->zStencilBufferFormats[0].format;
  lpComboBoxZStencilBuffer->ItemIndex = 0;
}

void SetupControls::refreshMultiSampleTypes()
{
  int i;

  lpDirect3DSetup->getMultiSampleTypes();

  lpComboBoxMultiSampleType->Items->Clear();

  for(i=0; i<lpDirect3DSetup->nrMultiSampleTypes; i++)
  {
    lpComboBoxMultiSampleType->Items->Add(
       lpDirect3DSetup->multiSampleTypes[i].name);
  }

  lpDirect3DSetup->multiSampleType = lpDirect3DSetup
       ->multiSampleTypes[0].type;
  lpComboBoxMultiSampleType->ItemIndex = 0;
}

void SetupControls::refreshMultiSampleQuality()
{
  int i;
  int nrQualityLevels = lpDirect3DSetup->nrQualityLevels[
                           lpComboBoxMultiSampleType->ItemIndex];
  lpComboBoxMultiSampleQuality->Items->Clear();

  for(i=0; i<int(nrQualityLevels);i++)
  {
    lpComboBoxMultiSampleQuality->Items->Add(
       AnsiString("Qualittsstufe ") + (i+1));
  }

  lpDirect3DSetup->qualityLevel = 0;
  lpComboBoxMultiSampleQuality->ItemIndex = 0;
}

void SetupControls::updateAdapter()
{
  lpDirect3DSetup->adapter = lpComboBoxAdapters->ItemIndex;
}

void SetupControls::updateMode()
{
  lpDirect3DSetup->mode = lpComboBoxModes->ItemIndex;
}

void SetupControls::updateType()
{
  if(lpRadioButtonsType[0]->Checked)
  {
    lpDirect3DSetup->deviceType = D3DDEVTYPE_HAL;
  }
  else
  {
    lpDirect3DSetup->deviceType = D3DDEVTYPE_REF;
  }
}

void SetupControls::updateWindowed()
{
  lpDirect3DSetup->windowed = lpRadioButtonsWindowed[1]->Checked;
}

void SetupControls::updateBackBufferFormat()
{
  lpDirect3DSetup->backBufferFormat =
     lpDirect3DSetup->backBufferFormats[lpComboBoxBackBuffer->ItemIndex].format;
}

void SetupControls::updateZStencilBufferFormat()
{
  lpDirect3DSetup->zStencilBufferFormat =
     lpDirect3DSetup->
        zStencilBufferFormats[lpComboBoxZStencilBuffer->
                              ItemIndex].format;
}

void SetupControls::updateMultiSampleType()
{
  lpDirect3DSetup->multiSampleType =
     lpDirect3DSetup->multiSampleTypes[
        lpComboBoxMultiSampleType->ItemIndex].type;
}

void SetupControls::updateMultiSampleQuality()
{
  lpDirect3DSetup->qualityLevel =
     lpComboBoxMultiSampleQuality->ItemIndex;
}

//-----------------------------------------------------------------------
// weitere Klassenmethoden frs Einstellungsfenster
//-----------------------------------------------------------------------

Direct3DParameters TFormDirect3DSetup::execute()
{
  Direct3DParameters parameters;

  lpSetupControls->init();
  ShowModal();

  parameters.adapter = lpDirect3DSetup->adapter;
  parameters.deviceType = lpDirect3DSetup->deviceType;
  parameters.windowed = lpDirect3DSetup->windowed;
  parameters.displayMode = lpDirect3DSetup->lpModes[lpDirect3DSetup->mode];
  parameters.backBufferFormat = lpDirect3DSetup->backBufferFormat;
  parameters.zStencilBufferFormat = lpDirect3DSetup->zStencilBufferFormat;
  parameters.multiSampleType = lpDirect3DSetup->multiSampleType;
  parameters.multiSampleQualityLevel = lpDirect3DSetup->qualityLevel;
  parameters.flags = D3DCREATE_HARDWARE_VERTEXPROCESSING;
  parameters.deviceCaps = lpDirect3DSetup->deviceCaps;

  return parameters;
}

//-----------------------------------------------------------------------
// Globale Methoden
//-----------------------------------------------------------------------

//(bislang keine!)

#endif
